# This file was automatically generated by SWIG (http://www.swig.org).
# Version 2.0.10
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.



from sys import version_info
if version_info >= (2,6,0):
    def swig_import_helper():
        from os.path import dirname
        import imp
        fp = None
        try:
            fp, pathname, description = imp.find_module('_algorithms', [dirname(__file__)])
        except ImportError:
            import _algorithms
            return _algorithms
        if fp is not None:
            try:
                _mod = imp.load_module('_algorithms', fp, pathname, description)
            finally:
                fp.close()
            return _mod
    _algorithms = swig_import_helper()
    del swig_import_helper
else:
    import _algorithms
del version_info
try:
    _swig_property = property
except NameError:
    pass # Python < 2.2 doesn't have 'property'.
def _swig_setattr_nondynamic(self,class_type,name,value,static=1):
    if (name == "thisown"): return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name,None)
    if method: return method(self,value)
    if (not static):
        self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)

def _swig_setattr(self,class_type,name,value):
    return _swig_setattr_nondynamic(self,class_type,name,value,0)

def _swig_getattr(self,class_type,name):
    if (name == "thisown"): return self.this.own()
    method = class_type.__swig_getmethods__.get(name,None)
    if method: return method(self)
    raise AttributeError(name)

def _swig_repr(self):
    try: strthis = "proxy of " + self.this.__repr__()
    except: strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

try:
    _object = object
    _newclass = 1
except AttributeError:
    class _object : pass
    _newclass = 0


import gippy

def acca(*args, **kwargs):
  """
    acca(GeoImage arg1, std::string filename, float arg3, float arg4, int arg5=5, int arg6=10, 
        int arg7=4000) -> GeoImage

    Parameters:
        arg1: gip::GeoImage const &
        filename: std::string
        arg3: float
        arg4: float
        arg5: int
        arg6: int
        arg7: int

    """
  return _algorithms.acca(*args, **kwargs)

def fmask(*args, **kwargs):
  """
    fmask(GeoImage geoimg, std::string filename, int arg3=3, int arg4=5) -> GeoImage

    Parameters:
        geoimg: gip::GeoImage const &
        filename: std::string
        arg3: int
        arg4: int

    """
  return _algorithms.fmask(*args, **kwargs)

def cookie_cutter(*args, **kwargs):
  """
    cookie_cutter(vector_GeoImage geoimgs, std::string filename="", GeoFeature feature=gip::GeoFeature(), 
        bool crop=False, std::string proj="", float xres=1.0, float yres=1.0, 
        int interpolation=0, gip::dictionary options=gip::dictionary()) -> GeoImage

    Parameters:
        geoimgs: std::vector< gip::GeoImage,std::allocator< gip::GeoImage > > const &
        filename: std::string
        feature: gip::GeoFeature
        crop: bool
        proj: std::string
        xres: float
        yres: float
        interpolation: int
        options: gip::dictionary

    """
  return _algorithms.cookie_cutter(*args, **kwargs)

def kmeans(*args, **kwargs):
  """
    kmeans(GeoImage arg1, std::string arg2, unsigned int classes=5, unsigned int iterations=5, 
        float threshold=1.0, unsigned int num_random=500) -> GeoImage

    Parameters:
        arg1: gip::GeoImage const &
        arg2: std::string
        classes: unsigned int
        iterations: unsigned int
        threshold: float
        num_random: unsigned int

    """
  return _algorithms.kmeans(*args, **kwargs)

def indices(*args, **kwargs):
  """
    indices(GeoImage geoimg, svector products, std::string filename="") -> GeoImage

    Parameters:
        geoimg: gip::GeoImage const &
        products: std::vector< std::string,std::allocator< std::string > >
        filename: std::string

    """
  return _algorithms.indices(*args, **kwargs)

def linear_transform(*args, **kwargs):
  """
    linear_transform(GeoImage geoimg, CImg< float > coef, std::string filename) -> GeoImage

    Parameters:
        geoimg: gip::GeoImage const &
        coef: CImg< float >
        filename: std::string

    """
  return _algorithms.linear_transform(*args, **kwargs)

def pansharp_brovey(*args, **kwargs):
  """
    pansharp_brovey(GeoImage geoimg, GeoImage panimg, CImg< float > weights=CImg< float >(), std::string filename="") -> GeoImage

    Parameters:
        geoimg: gip::GeoImage const &
        panimg: gip::GeoImage const &
        weights: CImg< float >
        filename: std::string

    """
  return _algorithms.pansharp_brovey(*args, **kwargs)

def rxd(*args, **kwargs):
  """
    rxd(GeoImage geoimg, std::string filename="") -> GeoImage

    Parameters:
        geoimg: gip::GeoImage const &
        filename: std::string

    """
  return _algorithms.rxd(*args, **kwargs)

def spectral_statistics(*args, **kwargs):
  """
    spectral_statistics(GeoImage arg1, std::string filename="") -> GeoImage

    Parameters:
        arg1: gip::GeoImage const &
        filename: std::string

    """
  return _algorithms.spectral_statistics(*args, **kwargs)
# This file is compatible with both classic and new-style classes.





#!/usr/bin/env python

import os
from copy import deepcopy
import numpy as np
from numpy.testing import assert_array_equal
import gippy as gp
import unittest
import gippy.test as gpt


class GeoImageTests(unittest.TestCase):

    prefix = 'test-'

    def setUp(self):
        """ Configure options """
        gp.Options.set_verbose(1)
        gp.Options.set_chunksize(4.0)

    def test0_open(self):
        """ Open existing image """
        geoimg = gpt.get_test_image()
        self.assertEqual(geoimg.xsize(), 627)
        self.assertEqual(geoimg.ysize(), 603)

    def test1_create(self):
        """ Create single band image """
        fout = 'test.tif'
        geoimg = gp.GeoImage.create(fout, xsz=1000, ysz=1000)
        self.assertTrue(geoimg.xsize() == 1000)
        self.assertTrue(geoimg.ysize() == 1000)
        self.assertTrue(os.path.exists(fout))
        # test resolution
        res = geoimg.resolution()
        self.assertEqual(res.x(), 1.0/geoimg.xsize())
        self.assertEqual(res.y(), -1.0/geoimg.ysize())
        os.remove(fout)

    def test_read(self):
        """ Read multiband image """
        geoimg = gpt.get_test_image()
        arr = geoimg.read()
        self.assertEqual(geoimg.nbands(), arr.shape[0])
        # make sure x, y dimensions are same when reading single bands
        self.assertEqual(arr.shape[1:3], geoimg[0].read().shape)

    def test_read_random_pixels(self):
        """ Read random pixels """
        geoimg = gpt.get_test_image()
        arr = geoimg.read_random_pixels(1000)

    def test_uint16_read(self):
        """ read uint16 makes uint16 array """
        fout = 'test.tif'
        geoimg = gp.GeoImage.create(fout, dtype='uint16')
        self.assertTrue(os.path.exists(fout))
        arr = geoimg.read()
        self.assertEqual(str(arr.dtype), geoimg.type().string())
        os.remove(fout)

    def test_loop_through_bands(self):
        """ Check that GeoImage is iterable """
        geoimg = gpt.get_test_image()
        for band in geoimg:
            self.assertEqual(band.xsize(), geoimg.xsize())

    def test_select(self):
        """ Selection of bands from GeoImage """
        img1 = gpt.get_test_image()
        img2 = img1.select(['red', 'green', 'blue'])
        self.assertTrue(np.array_equal(img1['red'].read(), img2[0].read()))
        self.assertTrue(np.array_equal(img1['green'].read(), img2[1].read()))
        self.assertTrue(np.array_equal(img1['blue'].read(), img2[2].read()))

    def test_persistent_metadata(self):
        """ Writing metadata and check for persistence after reopening """
        fout = 'test-meta.tif'
        geoimg = gp.GeoImage.create(fout, xsz=1000, ysz=1000, nb=3)
        geoimg.set_bandnames(['red', 'green', 'blue'])
        geoimg.set_nodata(7)
        self.assertEqual(geoimg.bandnames()[0], 'red')
        geoimg = None
        # reopen
        geoimg = gp.GeoImage(fout)
        self.assertEqual(geoimg[0].nodata(), 7)
        self.assertEqual(list(geoimg.bandnames()), ['red', 'green', 'blue'])
        geoimg = None
        os.remove(fout)

    def test_create_image_with_gain(self):
        """ Create int image with floating point gain """
        fout = 'test-gain.tif'
        geoimg = gp.GeoImage.create(fout, xsz=1000, ysz=1000, dtype='int16')
        geoimg.set_gain(0.0001)
        arr = np.zeros((1000,1000)) + 0.0001
        arr[0:500,:] = 0.0002
        geoimg[0].write(deepcopy(arr))
        arrout = geoimg[0].read()
        np.testing.assert_array_equal(arr, arrout)
        os.remove(fout)

    def test_create_multiband(self):
        """ Create an RGB image """
        fout = 'test_3band.tif'
        geoimg = gp.GeoImage.create(fout, xsz=1000, ysz=1000, nb=3)
        geoimg.set_bandnames(['green', 'red', 'blue'])
        # test selection of bands
        geoimg2 = geoimg.select(["red"])
        self.assertTrue(geoimg2.nbands() == 1)
        self.assertTrue(geoimg["red"].description() == "red")
        geoimg = None
        geoimg2 = None
        os.remove(fout)

    def test_create_temp_file(self):
        """ Create a temp file that is deleted when last reference gone """
        fout = self.prefix + '_temp.tif'
        geoimg = gp.GeoImage.create(fout, xsz=1000, ysz=1000, nb=5, temp=True)
        self.assertTrue(os.path.exists(fout))
        # keep a band
        band = geoimg[1]
        geoimg = None
        # band still references file
        self.assertTrue(os.path.exists(fout))
        band = None
        # file should now have been deleted
        self.assertFalse(os.path.exists(fout))

    def test_create_autoname_temp(self):
        """ Create temp file with auto-generated filename """
        geoimg = gp.GeoImage.create(xsz=1000, ysz=1000, nb=3)
        fout = geoimg.filename()
        self.assertTrue(os.path.exists(fout))
        geoimg = None
        self.assertFalse(os.path.exists(fout))

    def test_autoscale(self):
        """ Auto scale each band in image """
        geoimg = gpt.get_test_image()
        for band in geoimg:
            self.assertTrue(band.min() != 1.0)
            self.assertTrue(band.max() != 255.0)
        geoimg2 = geoimg.autoscale(minout=1.0, maxout=255.0)
        for band in geoimg2:
            self.assertTrue(band.min() == 1)
            self.assertTrue(band.max() == 255)

    def test_overviews(self):
        """ Add overviews to an image """
        fout = 'test-overviews.tif'
        geoimg = gp.GeoImage.create(filename=fout, xsz=1000, ysz=1000, nb=2)
        fout = geoimg.filename()
        # add overviews
        geoimg.add_overviews() 
        # clear overviews
        geoimg.add_overviews(levels=[])
        self.assertFalse(os.path.exists(fout + '.ovr'))
        geoimg = None
        geoimg = gp.GeoImage(fout, False)
        geoimg.add_overviews()
        self.assertTrue(os.path.exists(fout + '.ovr'))
        os.remove(fout)
        os.remove(fout + '.ovr')

    def test_save(self):
        """ Save image as new image with different datatype """
        fout = 'test-byte.tif'
        geoimg = gpt.get_test_image().autoscale(1.0, 255.0).save(fout, 'uint8')
        geoimg = None
        geoimg = gp.GeoImage(fout)
        self.assertEqual(geoimg.type().string(), 'uint8')
        self.assertEqual(geoimg[0].min(), 1.0)
        self.assertEqual(geoimg[0].max(), 255.0)
        os.remove(fout)

    def test_save_with_gain(self):
        """ Save image with a gain, which should copy through """
        geoimg = gpt.get_test_image().select([2])
        geoimg.set_gain(0.0001)
        fout = 'test-savegain.tif'
        imgout = geoimg.save(fout)
        assert_array_equal(imgout.read(), geoimg.read())
        os.remove(fout)

    def test_warp(self):
        """ Warping image into another (blank) image """
        bbox = np.array([0.0, 0.0, 1.0, 1.0])
        # default image in EPSG:4326 that spans 1 degree
        geoimg = gp.GeoImage.create(xsz=1000, ysz=1000, nb=3, proj='EPSG:4326', bbox=bbox)
        # 3857, set resolution to 100 meters
        imgout = geoimg.warp(proj='EPSG:3857', xres=100.0, yres=100.0)
        self.assertTrue(os.path.exists(imgout.filename()))
        self.assertEqual(imgout.xsize(), 1114)
        self.assertEqual(imgout.ysize(), 1114)
        self.assertAlmostEqual(np.ceil(imgout.resolution().x()), 100.0)

    def test_real_warp(self):
        """ Warp real image to another projection """
        geoimg = gpt.get_test_image()
        fout = 'test-realwarp.tif'
        imgout = geoimg.warp(fout, proj='EPSG:4326', xres=0.0003, yres=0.0003)
        self.assertEqual(imgout.xsize(), 653)
        self.assertEqual(imgout.ysize(), 547)
        os.remove(fout)

    def test_warp_into(self):
        """ Warp real image into an existing image """
        geoimg = gpt.get_test_image().select([1])
        ext = geoimg.extent()
        bbox = np.array([ext.x0(), ext.y0(), ext.width(), ext.height()])
        imgout = gp.GeoImage.create('', geoimg.xsize(), geoimg.ysize(), 1, geoimg.srs(),
                                    bbox, geoimg.type().string());
        geoimg.warp_into(imgout)
        self.assertEqual(imgout.read().sum(), geoimg.read().sum())




import gippy
from stestdata import TestData


# TODO - download landsat test image if not already
def get_test_image(sensor='landsat8', name='', bands=[]):
    """ Get test image from sat-testdata """
    t = TestData(sensor)

    if name == '':
        name = t.names[0]

    dat = t.examples[name].values()
    if len(bands) > 0:
        dat = [d for d in dat if d['band_type'] in bands]
    else:
        # filter out pan band
        dat = [d for d in dat if d['band_type'] != 'pan']

    filenames = [d['path'] for d in dat]
    bandnames = [d['band_type'] for d in dat]

    geoimg = gippy.GeoImage(filenames)
    geoimg.set_bandnames(bandnames)
    geoimg.set_nodata(0)
    return geoimg


"""
#arr = np.array([[1, 2], [3, 4], [5, 6]])
arr = np.array([1, 2, 3, 4, 5, 6])

VerboseOut('Original Array')
print arr, arr.dtype, '\n'

VerboseOut('Array w/o casting')
outarr = gippy.test(arr)
print outarr, outarr.dtype, '\n'

dtypes = ['uint8', 'uint16', 'int16', 'uint32', 'int32', 'uint64', 'int64', 'float32', 'float64']
for dt in dtypes:
    VerboseOut('Array as %s' % dt)
    outarr = gippy.test(arr.astype(dt))
    print outarr, outarr.dtype, '\n'
"""



#!/usr/bin/env python

import os
import unittest
from copy import deepcopy
import numpy as np
import gippy as gp
import gippy.algorithms as alg
import gippy.test as gpt


class GeoAlgorithmsTests(unittest.TestCase):

    def setUp(self):
        gp.Options.set_verbose(1)

    def test_rxd(self):
        """ RX anamoly detector """
        geoimg = gpt.get_test_image().select(['red', 'green', 'blue'])
        fout = 'test-rxd.tif'
        rxd = alg.rxd(geoimg, filename=fout)
        self.assertEqual(rxd.bandnames()[0], "RXD")
        self.assertEqual(rxd.xsize(), geoimg.xsize())
        self.assertEqual(rxd.ysize(), geoimg.ysize())
        self.assertEqual(rxd.nbands(), 1)
        rxd = None
        os.remove(fout)

    def test_pansharpen(self):
        """ Pansharpen multispectral image with panchromatic image """
        geoimg = gpt.get_test_image().select(['red', 'green', 'blue', 'nir'])
        panimg = gpt.get_test_image(bands=['pan'])
        fout = 'test-pansharpen.tif'
        imgout = alg.pansharp_brovey(geoimg, panimg, filename=fout)
        self.assertAlmostEqual(imgout.resolution().x(), panimg.resolution().x(), places=1)
        self.assertAlmostEqual(imgout.resolution().y(), panimg.resolution().y(), places=1)
        self.assertEqual(imgout.nbands(), 4)
        os.remove(fout)
        geoimg = gpt.get_test_image().select(['red', 'green', 'blue'])
        imgout = alg.pansharp_brovey(geoimg, panimg, filename=fout)
        self.assertEqual(imgout.nbands(), 3)
        self.assertAlmostEqual(imgout.resolution().x(), panimg.resolution().x(), places=1)
        self.assertAlmostEqual(imgout.resolution().y(), panimg.resolution().y(), places=1)
        os.remove(fout)

    def test_cookiecutter(self):
        """ Create mosaic from multiple images (cookie cutter) """
        bbox1 = np.array([0.0, 0.0, 1.0, 1.0])
        geoimg1 = gp.GeoImage.create(xsz=1000, ysz=1000, bbox=bbox1)
        bbox2 = np.array([1.0, 0.0, 1.0, 1.0])
        geoimg2 = gp.GeoImage.create(xsz=1000, ysz=1000, bbox=bbox2)
        res = geoimg1.resolution()
        imgout = alg.cookie_cutter([geoimg1, geoimg2], xres=res.x(), yres=res.y())
        ext = imgout.extent()
        self.assertEqual(ext.x0(), 0.0)
        self.assertEqual(ext.y0(), 0.0)
        self.assertEqual(ext.width(), 2.0)
        self.assertEqual(ext.height(), 1.0)

    def test_cookiecutter_gain(self):
        """ Cookie cutter on int image with floating point gain """
        bbox = np.array([0.0, 0.0, 1.0, 1.0])
        geoimg = gp.GeoImage.create(xsz=1000, ysz=1000, bbox=bbox, dtype='int16')
        geoimg.set_gain(0.0001)
        arr = np.zeros((1000,1000)) + 0.0001
        arr[0:500,:] = 0.0002
        geoimg.write(deepcopy(arr))
        res = geoimg.resolution()
        imgout = alg.cookie_cutter([geoimg], xres=res.x(), yres=res.y())
        np.testing.assert_array_equal(arr, imgout.read())

    def test_cookiecutter_real(self):
        """ Cookie cutter on single real image """
        geoimg = gpt.get_test_image().select(['red']) #, 'green', 'blue'])
        vpath = os.path.join(os.path.dirname(__file__), 'vectors')
        # test with feature of different projection
        feature = gp.GeoVector(os.path.join(vpath, 'aoi1_epsg4326.shp'))
        extin = feature.extent()
        imgout = alg.cookie_cutter([geoimg], feature=feature[0], xres=0.0003, yres=0.0003)
        extout = imgout.extent()
        self.assertAlmostEqual(extout.x0(), extin.x0())
        self.assertAlmostEqual(extout.y0(), extin.y0())
        self.assertAlmostEqual(extout.x1(), extin.x1())
        self.assertAlmostEqual(extout.y1(), extin.y1())

    def test_cookiecutter_real_reproj(self):
        """ Test with different projection """
        geoimg = gpt.get_test_image().select(['red', 'green', 'blue'])
        vpath = os.path.join(os.path.dirname(__file__), 'vectors')
        feature = gp.GeoVector(os.path.join(vpath, 'aoi1_epsg32416.shp'))
        extin = feature.extent()
        # test extent matches feature
        imgout = alg.cookie_cutter([geoimg], feature=feature[0], xres=30.0, yres=30.0)
        extout = imgout.extent()
        self.assertAlmostEqual(extout.x0(), extin.x0())
        self.assertAlmostEqual(extout.y0(), extin.y0())
        self.assertAlmostEqual(extout.x1(), extin.x1())
        self.assertAlmostEqual(extout.y1(), extin.y1())

    def test_cookiecutter_real_crop(self):
        """ Test cookie cutter with cropping """
        geoimg = gpt.get_test_image().select(['red', 'green', 'blue'])
        vpath = os.path.join(os.path.dirname(__file__), 'vectors')
        feature = gp.GeoVector(os.path.join(vpath, 'aoi1_epsg32416.shp'))
        imgout = alg.cookie_cutter([geoimg], feature=feature[0], xres=30.0, yres=30.0, crop=True)
        extin = feature.extent()
        extout = imgout.extent()
        self.assertTrue(extout.x0() >= extin.x0())
        self.assertTrue(extout.y0() >= extin.y0())
        self.assertTrue(extout.x1() <= extin.x1())
        self.assertTrue(extout.y1() <= extin.y1())

    def test_ndvi(self):
        """ Calculate NDVI using gippy and apply colortable """
        geoimg = gpt.get_test_image()
        fout = 'test-ndvi.tif'
        imgout = alg.indices(geoimg, ['ndvi'])
        # add colorramp
        red = np.array([255, 0, 0])
        green = np.array([0, 255, 0])
        white = np.array([255, 255, 255])
        imgout[0] = imgout[0].scale(-1.0, 1.0, 1, 255)
        imgout = imgout.save(fout, dtype='byte')
        # add color ramp for negative values
        imgout[0].add_colortable(red, white, value1=0, value2=128)
        # add color ramp for positive values
        imgout[0].add_colortable(white, green, value1=128, value2=255)
        # TODO - actually test something here
        os.remove(fout)

    def test_ndvi_numpy(self):
        """ Calculate NDVI using numpy (for speed comparison) """
        geoimg = gpt.get_test_image()
        nodata = geoimg[0].nodata()
        red = geoimg['RED'].read().astype('double')
        nir = geoimg['NIR'].read().astype('double')
        ndvi = np.zeros(red.shape) + nodata
        inds = np.logical_and(red != nodata, nir != nodata)
        ndvi[inds] = (nir[inds] - red[inds])/(nir[inds] + red[inds])
        fout = 'test-ndvi2.tif'
        geoimgout = gp.GeoImage.create_from(geoimg, fout, dtype="float64")
        geoimgout[0].write(ndvi)
        geoimgout = None
        geoimg = None
        os.remove(fout)



#!/usr/bin/env python

import os
import gippy as gp
import unittest
import gippy.test as gpt

"""
    GeoResource is a virtual base class, and thus cannot be instanatiated on it's own.
    It is created with GeoImage or GeoRaster.  This doesn't test GeoResource on it's own,
    rather, it uses GeoImage, but tests only the functions of the base GeoResource class
"""


class GeoResourceTests(unittest.TestCase):

    def setUp(self):
        """ Configure options """
        gp.Options.set_verbose(1)
        gp.Options.set_chunksize(4.0)

    def test_filename(self):
        """ Check filename, basename, extension functions """
        fout = 'test.tif'
        geoimg = gp.GeoImage.create(fout)
        self.assertTrue(os.path.exists(fout))
        self.assertEqual(geoimg.filename(), fout)
        self.assertEqual(geoimg.basename(), 'test')
        self.assertEqual(geoimg.extension(), 'tif')
        os.remove(fout)

    def test_format(self):
        """ Get and set file format and verify """
        dformat = gp.Options.defaultformat()
        gp.Options.set_defaultformat('GTiff')
        geoimg = gp.GeoImage.create()
        self.assertEqual(geoimg.extension(), 'tif')
        # change default format
        gp.Options.set_defaultformat('MEM')
        geoimg = gp.GeoImage.create()
        self.assertEqual(geoimg.extension(), '')
        # create by specifying format to create
        geoimg = gp.GeoImage.create(format='GTiff')
        self.assertEqual(geoimg.extension(), 'tif')
        # change back to original format
        gp.Options.set_defaultformat(dformat)

    def test_size(self):
        """ Retrieve size and dimension in pixels """
        geoimg = gp.GeoImage.create(xsz=500, ysz=1500)
        self.assertEqual(geoimg.xsize(), 500)
        self.assertEqual(geoimg.ysize(), 1500)
        self.assertEqual(geoimg.size(), 1500*500)

    def test_coordinates(self):
        """ Validate coordinates of pixel locations """
        geoimg = gp.GeoImage.create(xsz=1000, ysz=1000)
        pt = geoimg.geoloc(0, 0)
        self.assertEqual(pt.x(), 0.0)
        self.assertEqual(pt.y(), 1.0)
        # this is out of bounds WRT pixels, but is coordinate of lower right corner
        # of the last pixel whereas geoloc(999, 999) is top left corner of last pixel
        pt = geoimg.geoloc(1000, 1000)
        self.assertEqual(pt.x(), 1.0)
        self.assertEqual(pt.y(), 0.0)
        self.assertEqual(geoimg.minxy().x(), 0.0)
        self.assertEqual(geoimg.minxy().x(), 0.0)
        self.assertEqual(geoimg.maxxy().x(), 1.0)
        self.assertEqual(geoimg.maxxy().x(), 1.0)
        # test resolution
        self.assertEqual(geoimg.resolution().x(), 1.0/1000.0)
        self.assertEqual(geoimg.resolution().y(), -1.0/1000.0)
        # test extent
        extent = geoimg.extent()
        self.assertEqual(extent.x0(), 0.0)
        self.assertEqual(extent.y0(), 0.0)
        self.assertEqual(extent.x1(), 1.0)
        self.assertEqual(extent.y1(), 1.0)

    def test_affine(self):
        """ Compare affine vs coordinates and resolution """
        geoimg = gp.GeoImage.create(xsz=100, ysz=100)
        aff = geoimg.affine()
        self.assertEqual(len(aff), 6)
        self.assertEqual(aff[0], geoimg.minxy().x())
        self.assertEqual(aff[1], geoimg.resolution().x())
        self.assertEqual(aff[2], 0.0)
        self.assertEqual(aff[3], geoimg.maxxy().y())
        self.assertEqual(aff[4], 0.0)
        self.assertEqual(aff[5], geoimg.resolution().y())
        # test with real image
        geoimg = gpt.get_test_image()
        aff = geoimg.affine()
        self.assertEqual(len(aff), 6)
        self.assertEqual(aff[0], geoimg.minxy().x())
        self.assertEqual(aff[1], geoimg.resolution().x())
        self.assertEqual(aff[2], 0.0)
        self.assertEqual(aff[3], geoimg.maxxy().y())
        self.assertEqual(aff[4], 0.0)
        self.assertEqual(aff[5], geoimg.resolution().y())

    def test_spatialreference(self):
        """ Set spatial reference """
        geoimg = gp.GeoImage.create(xsz=100, ysz=100)
        prj = geoimg.srs()
        geoimg.set_srs('EPSG:4326')
        # set it to default above, so they should be same
        self.assertEqual(geoimg.srs(), prj)

    def test_chunks(self):
        """ Chunk image to create list of rects """
        geoimg = gp.GeoImage.create(xsz=2000, ysz=2000)
        # test with one chunk
        chunks = geoimg.chunks(numchunks=1)
        self.assertEqual(len(chunks), 1)
        self.assertEqual(chunks[0].width(), geoimg.xsize())
        self.assertEqual(chunks[0].height(), geoimg.ysize())
        # test with 100 chunks
        chunks = geoimg.chunks(numchunks=100)
        # test height of chunks is the same (except last one)
        self.assertEqual(len(chunks), 100)
        for i in range(1, len(chunks)-1):
            self.assertEqual(chunks[i].height(), chunks[i-1].height())
            self.assertEqual(chunks[i].width(), geoimg.xsize())
            self.assertEqual(chunks[i].y0(), chunks[i-1].y1())

    def test_meta(self):
        """ Set and retrieve metadata """
        geoimg = gp.GeoImage.create(xsz=1000, ysz=1000)
        meta = geoimg.meta()
        self.assertEqual(len(meta), 0)
        # set single metadata item
        geoimg.add_meta('testkey', 'testvalue')
        self.assertEqual(geoimg.meta('testkey'), 'testvalue')
        md = {'key1': 'val1', 'key2': 'val2', 'key3': 'val3'}
        geoimg.add_meta(md)
        md['testkey'] = 'testvalue'
        md2 = geoimg.meta()
        for m in md:
            self.assertEqual(geoimg.meta(m), md[m])
            self.assertEqual(md2[m], md[m])



#!/usr/bin/env python
################################################################################
#    GIPPY: Geospatial Image Processing library for Python
#
#    AUTHOR: Matthew Hanson
#    EMAIL:  matt.a.hanson@gmail.com
#
#    Copyright (C) 2015 Applied Geosolutions
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
################################################################################
"""
setup for GIP and gippy
"""

import os
import sys
import glob
import re
import subprocess
import logging
import shutil
from numpy import get_include as numpy_get_include
from imp import load_source

# setup imports
from setuptools import setup, Extension
from setuptools.command.build_ext import build_ext
from setuptools.command.install import install
from setuptools.command.develop import develop
#from wheel.bdist_wheel import bdist_wheel
from distutils import sysconfig

__version__ = load_source('gippy.version', 'gippy/version.py').__version__

# get the dependencies and installs
with open('requirements.txt') as fid:
    install_requires = [l.strip() for l in fid.readlines() if l]

with open('requirements-dev.txt') as fid:
    test_requires = [l.strip() for l in fid.readlines() if l]

# logging
logging.basicConfig(level=logging.DEBUG)
# logging.basicConfig()
log = logging.getLogger(os.path.basename(__file__))


class CConfig(object):
    """Interface to config options from any utility"""

    def __init__(self, cmd):
        self.cmd = cmd
        self.get_include()
        self.get_libs()

    def get(self, option):
        try:
            stdout, stderr = subprocess.Popen(
                [self.cmd, option],
                stdout=subprocess.PIPE, stderr=subprocess.PIPE).communicate()
        except OSError:
            # e.g., [Errno 2] No such file or directory
            raise OSError("Could not find script")
        if stderr and not stdout:
            raise ValueError(stderr.strip())
        if sys.version_info[0] >= 3:
            result = stdout.decode('ascii').strip()
        else:
            result = stdout.strip()
        # log.info('%s %s: %r', self.cmd, option, result)
        return result

    def get_include(self):
        self.include = []
        for item in self.get('--cflags').split():
            if item.startswith("-I"):
                self.include.extend(item[2:].split(":"))
        return self.include

    def get_libs(self):
        self.libs = []
        self.lib_dirs = []
        self.extra_link_args = []
        for item in self.get('--libs').split():
            if item.startswith("-L"):
                self.lib_dirs.extend(item[2:].split(":"))
            elif item.startswith("-l"):
                self.libs.append(item[2:])
            else:
                # e.g. -framework GEOS
                self.extra_link_args.append(item)

    def version(self):
        match = re.match(r'(\d+)\.(\d+)\.(\d+)', self.get('--version').strip())
        return tuple(map(int, match.groups()))


class _build_ext(build_ext):
    # builds the external modules: libgip.so, _gippy.so, _algorithms.so
    def run(self):
        log.debug('_build_ext run')
        for m in swig_modules:
            # know where to find libgip for linking
            m.library_dirs.append(os.path.join(self.build_lib, 'gippy'))
            # on linux add to rpath
            if sys.platform != 'darwin':
                m.runtime_library_dirs.append('$ORIGIN')

        # in python3 the created .so files have names tagged to version, see PEP 3147, 3149
        if sysconfig.get_config_var('SOABI') is not None:
            # create link
            libpath = os.path.join(self.build_lib, 'gippy')
            if not os.path.exists(libpath):
                os.makedirs(libpath)
            link = os.path.join(self.build_lib, gip_module._full_name + '.so')
            os.symlink(os.path.basename(gip_module._file_name), link)

        # build the extensionss
        build_ext.run(self)

        # for mac update runtime library location. Use otool -L to see shared libs in a .so
        if sys.platform == 'darwin':
            old_name = os.path.join(self.build_lib, gip_module._file_name)
            new_name = '@loader_path/libgip.so'  # %s' % os.path.basename(gip_module._file_name)
            cmd0 = 'install_name_tool -change %s %s' % (old_name, new_name)
            for m in swig_modules:
                cmd = '%s %s' % (cmd0, os.path.join(os.path.abspath(self.build_lib), m._file_name))
                log.debug(cmd)
                out = subprocess.check_output(cmd.split(' '))
                # also update top level .so files 'develop' likes to create (still not sure why)
                if os.path.exists(os.path.basename(m._file_name)):
                    cmd = '%s %s' % (cmd0, os.path.basename(m._file_name))
                    log.debug(cmd)
                    out = subprocess.check_output(cmd.split(' '))
                log.debug(out)


class _develop(develop):
    def run(self):
        log.debug('_develop run')
        develop.run(self)
        # move lib files into gippy directory
        [shutil.move(f, 'gippy/') for f in glob.glob('*.so')]
        if sysconfig.get_config_var('SOABI') is not None:
            # rename libgip if Python 3.2+
            os.rename(gip_module._file_name, 'gippy/libgip.so')


class _install(install):
    def run(self):
        log.debug('_install run')
        # ensure extension built before packaging
        self.run_command('build_ext')
        install.run(self)


#class _bdist_wheel(bdist_wheel):
    # binary wheel
#    def run(self):
#        log.debug('_bdist_wheel run')
#        self.distribution.ext_modules = [gip_module] + swig_modules
#        self.run_command('build_ext')
#        bdist_wheel.run(self)


# GDAL config parameters
gdal_config = CConfig(os.environ.get('GDAL_CONFIG', 'gdal-config'))

extra_compile_args = ['-fPIC', '-O3', '-std=c++11']

if gdal_config.version()[0] == 2:
    extra_compile_args.append('-D GDAL2')

extra_link_args = gdal_config.extra_link_args

# not sure if current directory is necessary here
lib_dirs = gdal_config.lib_dirs + ['./']

if sys.platform == 'darwin':
    extra_compile_args.append('-stdlib=libc++')
    extra_link_args.append('-stdlib=libc++')

    ldshared = sysconfig.get_config_var('LDSHARED')

    sysconfig._config_vars['LDSHARED'] = re.sub(
        ' +', ' ',
        ldshared.replace('-bundle', '-dynamiclib')
    )

    extra_compile_args.append('-mmacosx-version-min=10.8')
    extra_link_args.append('-mmacosx-version-min=10.8')
    # silence various warnings
    extra_compile_args.append('-Wno-absolute-value')
    extra_compile_args.append('-Wno-shift-negative-value')
    extra_compile_args.append('-Wno-parentheses-equality')
    extra_compile_args.append('-Wno-deprecated-declarations')
else:
    # Remove the "-Wstrict-prototypes" compiler option that swig adds, which isn't valid for C++.
    cfg_vars = sysconfig.get_config_vars()
    for key, value in cfg_vars.items():
        if type(value) == str:
            cfg_vars[key] = value.replace("-Wstrict-prototypes", "")
    extra_compile_args.append('-Wno-maybe-uninitialized')

# the libgip.so module containing all the C++ code
gip_module = Extension(
    name=os.path.join("gippy", "libgip"),
    sources=glob.glob('GIP/*.cpp'),
    include_dirs=['GIP', numpy_get_include()] + gdal_config.include,
    library_dirs=lib_dirs,
    libraries=[
        'pthread'
    ] + gdal_config.libs,
    extra_compile_args=extra_compile_args,
    extra_link_args=extra_link_args
)

# the swig .so modules containing the C++ code that wraps libgip.so
swig_modules = []
for n in ['gippy', 'algorithms']:
    src = os.path.join('gippy', n + '.i')
    cppsrc = os.path.join('gippy', n + '_wrap.cpp')
    src = cppsrc if os.path.exists(cppsrc) else src
    swig_modules.append(
        Extension(
            name=os.path.join('gippy', '_' + n),
            sources=[src],
            swig_opts=['-c++', '-w509', '-w511', '-w315', '-IGIP', '-fcompact', '-fvirtual', '-keyword'],
            include_dirs=['GIP', numpy_get_include()] + gdal_config.include,
            library_dirs=lib_dirs,
            libraries=[
                'gip', 'pthread'
            ] + gdal_config.libs,
            extra_compile_args=extra_compile_args,
            extra_link_args=extra_link_args
        )
    )


setup(
    name='gippy',
    version=__version__,
    description='Geospatial Image Processing for Python',
    author='Matthew Hanson',
    author_email='matt.a.hanson@gmail.com',
    license='Apache v2.0',
    # platform_tag='linux_x86_64',
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: C++',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5'
    ],
    ext_modules=[gip_module] + swig_modules,
    packages=['gippy'],
    install_requires=install_requires,
    test_suite='nose.collector',
    tests_require=test_requires,
    cmdclass={
        "build_ext": _build_ext,
        "develop": _develop,
        "install": _install,
        #"bdist_wheel": _bdist_wheel,
    }
)



#!/usr/bin/env python

import os
import numpy as np
import gippy as gp
import unittest
import gippy.test as gpt
# from nose.tools import raises

"""
Included are some tests for doing processing in NumPy instead of Gippy,
for doing speed comparisons. To see the durations of each test use:
    $ nosetests test --with-timer -v
"""


class GeoRasterTests(unittest.TestCase):
    """ Speed tests vs NumPy """

    def setUp(self):
        """ Configure options """
        gp.Options.set_verbose(1)
        gp.Options.set_chunksize(256.0)

    def test_size(self):
        """ Retrieve size and dimension in pixels """
        # note that xsize and ysize are redefined in GeoRaster from
        # GeoResource, thus it is tested again
        geoimg = gp.GeoImage.create(xsz=500, ysz=1500)
        self.assertEqual(geoimg.xsize(), 500)
        self.assertEqual(geoimg.ysize(), 1500)
        self.assertEqual(geoimg.size(), 1500*500)

    def test_type(self):
        """ Set datatype on create and verify """
        geoimg = gp.GeoImage.create(dtype='uint32')
        self.assertEqual(geoimg.type().string(), 'uint32')

    def test_naming(self):
        """ Get basename and desription """
        fout = 'test-image.tif'
        bname = os.path.splitext(fout)[0]
        bandnames = ['red', 'green', 'blue']
        geoimg = gp.GeoImage.create(fout, nb=3)
        geoimg.set_bandnames(bandnames)
        for i in range(0, 3):
            self.assertEqual(geoimg[i].description(), bandnames[i])
            self.assertEqual(geoimg[i].basename(), '%s[%s]' % (bname, i))
        os.remove(fout)
        # TODO - test color

    def test_gain_and_offset(self):
        """ Set and retrieve gain and offset """
        fout = 'test-gainoffset.tif'
        gains = [2.0, 3.0]
        offsets = [4.0, 5.0]
        geoimg = gp.GeoImage.create(fout, nb=2)
        geoimg[0].set_gain(gains[0])
        geoimg[1].set_gain(gains[1])
        geoimg[0].set_offset(offsets[0])
        geoimg[1].set_offset(offsets[1])
        # check persistance
        geoimg = None
        geoimg = gp.GeoImage(fout)
        for i in range(0, 2):
            self.assertEqual(geoimg[i].gain(), gains[i])
            self.assertEqual(geoimg[i].offset(), offsets[i])
        os.remove(fout)

    def test_nodata(self):
        """ Set nodata and retrieve """
        fout = 'test-nodata.tif'
        geoimg = gp.GeoImage.create(fout, xsz=100, ysz=100)
        geoimg.set_nodata(1)
        self.assertEqual(geoimg[0].nodata(), 1)
        geoimg = None
        geoimg = gp.GeoImage(fout)
        self.assertEqual(geoimg[0].nodata(), 1)
        # check that entire array is nan
        arr = np.where(geoimg.read() == np.nan)
        self.assertEqual(len(arr[0]), 0)
        self.assertEqual(len(arr[1]), 0)
        os.remove(fout)

    def test_bandmeta(self):
        """ Set metadata on band and retrieve """
        fout = 'test-meta.tif'
        geoimg = gp.GeoImage.create(fout, xsz=100, ysz=100)
        geoimg[0].add_bandmeta('TESTKEY', 'TESTVALUE')
        geoimg = None
        geoimg = gp.GeoImage(fout)
        self.assertEqual(geoimg[0].bandmeta('TESTKEY'), 'TESTVALUE')
        os.remove(fout)

    # TODO - test masking

    def test_stats(self):
        """ Calculate statistics using gippy """
        geoimg = gpt.get_test_image()
        for band in geoimg:
            stats = band.stats()
            mask = band.data_mask() == 1
            # check against numpy
            arr = band.read()
            self.assertAlmostEqual(arr[mask].min(), stats[0])
            self.assertAlmostEqual(arr[mask].max(), stats[1])
            self.assertAlmostEqual(arr[mask].mean(), stats[2], places=2)

    def test_scale(self):
        """ Scale image to byte range """
        geoimg = gpt.get_test_image()
        for band in geoimg:
            band = band.autoscale(minout=1, maxout=255, percent=2.0)
            self.assertTrue(band.min() == 1)
            self.assertTrue(band.max() == 255)

    def test_histogram(self):
        """ Calculate histogram of blank data """
        geoimg = gp.GeoImage.create(xsz=10, ysz=10, nb=2)
        arr = np.arange(10).reshape(1, 10) + 1
        for i in range(9):
            arr = np.append(arr, arr, axis=0)
        geoimg[0].write(arr.astype('uint8'))
        hist = geoimg[0].histogram(bins=10, normalize=False)
        self.assertEqual(hist[0], 10)
        self.assertEqual(hist.sum(), geoimg.size())
        hist = geoimg[0].histogram(bins=10)
        self.assertAlmostEqual(hist.sum(), 1.0)
        self.assertAlmostEqual(hist[0], 0.1)
        hist = geoimg[0].histogram(bins=10, normalize=False, cumulative=True)
        self.assertAlmostEqual(hist[-1], geoimg.size())

    def test_real_histogram(self):
        """ Calculate histogram of real data """
        geoimg = gpt.get_test_image()
        hist = geoimg[0].histogram(normalize=False)
        self.assertEqual(len(hist), 100)
        self.assertEqual(hist.sum(), geoimg.size())

    def test_sqrt(self):
        """ Calculate sqrt of image """
        geoimg = gpt.get_test_image().select(['red', 'green', 'swir1', 'nir'])
        for band in geoimg:
            vals = band.sqrt().read()
            mask = band.data_mask() == 1
            # check against numpy
            arr = band.read()
            self.assertTrue((vals[mask] == np.sqrt(arr[mask])).any())

    # TODO - test processing functions
    # Test filters
    def test_laplacian(self):
        """ Test with laplacian filter """
        geoimg = gp.GeoImage.create(xsz=10, ysz=10)
        arr = geoimg.read()
        arr[:, 0:6] = 1
        geoimg[0].write(arr)
        arrout = geoimg[0].laplacian().read()
        self.assertEqual(arrout[0, 5], -1.)
        self.assertEqual(arrout[0, 6], 1.)

    def test_convolve(self):
        """ Convolve an image with a 3x3 kernel """
        geoimg = gp.GeoImage.create(xsz=10, ysz=10)
        arr = geoimg.read() + 1
        geoimg[0].write(arr)
        kernel = np.array([[1, 1, 1], [1, 1, 1], [1, 1, 1]])
        arrout = geoimg[0].convolve(kernel, boundary=False).read()
        self.assertEqual(arrout[0, 0], 4)
        self.assertEqual(arrout[5, 5], 9)
        self.assertEqual(arrout[5, 0], 6)

    def test_skeletonize(self):
        """ Skeletonize a binary imager """
        geoimg = gp.GeoImage.create(xsz=10, ysz=10)
        arr = geoimg.read()
        arr[3:8, :] = 1
        geoimg[0].write(arr)
        arrout = geoimg[0].skeletonize().read()

    def test_write(self):
        """ Write arrays of different datatype """
        geoimg = gp.GeoImage.create(xsz=100, ysz=100, dtype='uint8')
        arr = np.ones((100, 100)).astype('uint8')
        geoimg[0].write(arr)
        self.assertTrue(np.array_equal(arr, geoimg[0].read()))
        arr = np.ones((100, 100)).astype('float32')
        geoimg[0].write(arr)
        self.assertTrue(np.array_equal(arr, geoimg[0].read()))

    """
    def test_invalid_args(self):
        # Check that invalid arguments throw error
        geoimg = gippy.GeoImage.create(xsz=100, ysz=100, dtype='uint8')
        try:
            geoimg[0].write('invalid arg')
            geoimg[0].write([1.0, 1.0])
            self.assertTrue(False)
        except:
            pass
    """
